! Sigmoid -- Make sigmoid reflectivity model.
!
!	Sigmoid [n1=400 n2=100 o1=0 d1=.004 o2=0 d2=.032 ] > out.H
!%
use sep
use random_mod

implicit none
integer 				:: large, n1, n2, i1, i2, it, is
real, dimension (:), allocatable	:: imp1, imp2
real, dimension (:,:), allocatable	:: dipper, earth, refl, &
					   sig1, sig2, fault
real					:: o1, d1, o2, d2, rand, t, frac

call sep_init ()
call from_either ("n1",n1,400)
call from_either ("n2",n2,100)

call to_history ("n3",1)
call to_history ("esize",4)

call from_either ("large",large,5*n1)

call from_par ("o1",o1,0.  ) ; call from_par ("o2",o2,0.)
call from_par ("d1",d1,.004) ; call from_par ("d2",d2,.032)

call to_history ("label1","sec")
call to_history ("label2","km")
call sep_close ()

allocate (imp1(large), imp2(large))
allocate (dipper(n1,n2), earth(n1,n2), refl(n1,n2))
allocate (sig1(n1,n2), sig2(n1,n2), fault(n1,n2))

call random_init (19921995)
imp1(1) = 1.
do i1= 2, large 
	call random (rand)
	if (rand > .2) then
 		imp1(i1) = imp1(i1-1)
	else	
		call random (rand)
		imp1(i1) = 1. + .1 * rand
	end if
end do

imp2(1) = 1.
do i1= 2, large
	call random (rand)
	if (rand > .3) then
		imp2(i1) = imp2(i1-1)
	else	
		call random (rand)
		imp2(i1) = 1. + .1 * rand
	end if
end do

do i2= 1, n2
do i1= 1, n1
	t = i1 + i2 * .3 * (1.*n1)/(1.*n2)
	it = t
	frac = t - it
        if (it >= 1 .and. it+1 <= large) then
           dipper (i1, i2) = imp1 (it) * (1.-frac) + imp1 (it+1) * frac
        else
           dipper (i1, i2) = 0.
        end if
end do
end do

do i2= 1, n2
do i1= 1, n1 
	t = i1 + n1 *.15 * ( 1.1 + cos( ( 3 *3.14 * i2)/n2) )
	it = t
	is = it + 10
	frac = t - it
        if (it >= 1 .and. it+1 <= large .and. &
            is >= 1 .and. is+1 <= large) then
           sig1 (i1, i2) = imp2 (it) * (1.-frac) + imp2 (it+1) * frac
           sig2 (i1, i2) = imp2 (is) * (1.-frac) + imp2 (is+1) * frac
        else
           sig1 (i1, i2) = 0.
           sig2 (i1, i2) = 0.
        end if
end do
end do

do i2= 1, n2 					! fault
do i1= 1, n1 
	if ((float(i2)/n2-.9) < -.75*((i1+1.)/n1)**2) then
		fault(i1,i2) = sig1(i1,i2)
	else
		fault(i1,i2) = sig2(i1,i2)
	end if
end do
end do

do i2= 1, n2 
do i1= 1, n1 
	it = i1 + i2 * .3 * (1.*n1)/(1.*n2)
	if (it > n1/2) then
		earth(i1,i2) = fault(i1,i2) 
	else	 
		earth(i1,i2) = dipper (i1,i2)
	end if
end do
end do

refl (1,:) = 0.
refl (2 :, :) = (earth(: n1-1,:) - earth(2 :,:)) / &	
		(earth(: n1-1,:) + earth(2 :,:))

do i2= 1, 10
	refl (:,      i2    ) = refl (:,      i2    ) * i2 / 10.
	refl (:, n2 - i2 + 1) = refl (:, n2 - i2 + 1) * i2 / 10.
end do

do i2= 1, 5
	refl (:,      i2    ) = refl (:,      i2    ) * i2 / 5.
	refl (:, n2 - i2 + 1) = refl (:, n2 - i2 + 1) * i2 / 5.
end do

call sep_write (refl)

deallocate (imp1, imp2, dipper, earth, refl, sig1, sig2, fault)

call exit (0)
end

