!
! NAME
!       Log - Base ten logarithm of absolute values.
!
! SYNOPSIS
!       <in.H  Log  [range=3 centered=2] > out.H
!
! PARAMETERS
!   from par:
!   	range           defines the smallest allowed value.
!                       Defining max to be the maximum of the logs (best 
!                       signal), output values (bad signal) below (max-range) 
!                       are set to (max-range). Often you might choose range
!                       to be a small integer, for example, range=3 means
!                       each input handled as max(input, biggest/1000).
!
!       centered        defines method of shifting mean of log.
!       if ( centered == 0 )                            ! graph
!                       The max is subtracted so all logs are negative.
!                       label2="log base 10" is putched
!       if ( centered == 1)                             ! wiggle and taplot
!                       The average of max and (max-range) is subtracted.
!                       (effectively scaling the original data)
!                       pclip=100 is putched.
!       if ( centered == 2)                             ! tomography
!                       The average value of all the logs is subtracted.
!                       range=2*(max-avg) is putched.
!
! KEYWORDS      log logarithm
!
! WHERE
!       ./cube/seis/Log.rs
!
!%
! author:	Claerbout and Harlan	8/9/89
! jon 8-10-93	added tomography centering.  set default range=3
use sep

implicit none
integer 							:: n1, n2, n3, centered
real, dimension (:), allocatable	:: x, g
real								:: range, scale, big, avg

call sep_init ()
call from_history (n1, n2, n3)

allocate (x(n1*n2*n3), g(n1*n2*n3))

call sep_read (x)
x = abs (x)
scale = alog(10.)
g = alog (x +1.e-35) / scale

call from_par ("range",range,3.)
big = maxval (g)
where (g < big-range) g =  big-range

call from_par ("centered",centered,2)
select case (centered)
case (0) 
	g = g - big
	call to_history ("label2", "log base 10")
case (1) 
	avg = (big + (big-range)) / 2.
	g = g - avg
	call to_history ("pclip",100)
case (2)
	avg = sum (g) / size (g)
	g = g - avg
	call to_history ("range", 2.*(big-avg) )
end select

call sep_close()
call sep_write (g)

call exit (0)
end
